// WL_AGENT.C

#include "WL_DEF.H"
#pragma hdrstop


/*
=============================================================================

						 LOCAL CONSTANTS

=============================================================================
*/

#define MAXMOUSETURN	10


#define MOVESCALE		150l
#define BACKMOVESCALE	100l
#define ANGLESCALE		20

/*
=============================================================================

						 GLOBAL VARIABLES

=============================================================================
*/



//
// player state info
//
boolean		running;
long		thrustspeed;

unsigned	plux,pluy;			// player coordinates scaled to unsigned

int			anglefrac;
int			gotgatgun;	// JR

objtype		*LastAttacker;

/*
=============================================================================

						 LOCAL VARIABLES

=============================================================================
*/

extern   statetype s_fire1;
extern   statetype s_rocket;

void	T_Player (objtype *ob);
void	T_Attack (objtype *ob);

statetype s_player = {false,0,0,T_Player,NULL,NULL};
statetype s_attack = {false,0,0,T_Attack,NULL,NULL};


long	playerxmove,playerymove;

struct atkinf
{
	char	tics,attack,frame;		// attack is 1 for weapons using bullets, 2 for knife
} attackinfo[6][4] =

{
{ {6,0,1},{6,2,2},{6,0,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,0,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,3,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,4,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,4,3},{6,-1,4} },
{ {6,0,1},{6,1,2},{6,0,3},{6,-1,4} },
};


int	strafeangle[9] = {0,90,180,270,45,135,225,315,0};

void    DrawWeapon (void);
void    GiveWeapon (int weapon);
void	GiveAmmo (int ammo);
void	GiveFuel (int fuel);
void	GiveRocket (int rocket);

//===========================================================================

//----------

void Attack (void);
void Use (void);
void Search (objtype *ob);
void SelectWeapon (void);
void SelectItem (void);

//----------

boolean TryMove (objtype *ob);
void T_Player (objtype *ob);

void ClipMove (objtype *ob, long xmove, long ymove);

/*
=============================================================================

						CONTROL STUFF

=============================================================================
*/

int CheckAmmo (int weapon)
{
   switch (weapon)
   {
      case wp_knife:
          return 1;
      case wp_pistol:
      case wp_machinegun:
      case wp_chaingun:
           if (gamestate.ammo > 0)
           return 1;
         break;
      case wp_flamethrower:
           if (gamestate.fuel > 0)
           return 1;
         break;
      case wp_rocketlauncher:
           if (gamestate.rockets > 0)
           return 1;
         break;
   }
   return 0;
}

/*
======================
=
= CheckWeaponChange
=
= Keys 1-6 change weapons
=
======================
*/

void CheckWeaponChange (void)
{
    int i, kbuttons[6]={sc_1,sc_2,sc_3,sc_4,sc_5,sc_6};

    for (i=0; i<NUMWEAPONS; i++)
    {
		if ((Keyboard[kbuttons[i]]) && (i != gamestate.weapon))
		{
			if (gamestate.bestweapon & (1<<i) && CheckAmmo(i))
			{
				gamestate.weapon = gamestate.chosenweapon = i;
				if(gamestate.chosenweapon != 0)
				LastScan = 0;
				DrawAmmo();
			}
		}
	}
	DrawWeapon();
}


/*
=======================
=
= ControlMovement
=
= Takes controlx,controly, and buttonstate[bt_strafe]
=
= Changes the player's angle and position
=
= There is an angle hack because when going 70 fps, the roundoff becomes
= significant
=
=======================
*/

void ControlMovement (objtype *ob)
{
	long	oldx,oldy;
	int		angle,maxxmove;
	int		angleunits;
	long	speed;

	thrustspeed = 0;

	oldx = player->x;
	oldy = player->y;

//
// side to side move
//
	if (buttonstate[bt_strafe])
	{
	//
	// strafing
	//
	//
		if (controlx > 0)
		{
			angle = ob->angle - ANGLES/4;
			if (angle < 0)
				angle += ANGLES;
			Thrust (angle,controlx*MOVESCALE);	// move to left
		}
		else if (controlx < 0)
		{
			angle = ob->angle + ANGLES/4;
			if (angle >= ANGLES)
				angle -= ANGLES;
			Thrust (angle,-controlx*MOVESCALE);	// move to right
		}
	}
	else
	{
	//
	// not strafing
	//
		anglefrac += controlx;
		angleunits = anglefrac/ANGLESCALE;
		anglefrac -= angleunits*ANGLESCALE;
		ob->angle -= angleunits;

		if (ob->angle >= ANGLES)
			ob->angle -= ANGLES;
		if (ob->angle < 0)
			ob->angle += ANGLES;

	}

//
// forward/backwards move
//
	if (controly < 0)
	{
		Thrust (ob->angle,-controly*MOVESCALE);	// move forwards
	}
	else if (controly > 0)
	{
		angle = ob->angle + ANGLES/2;
		if (angle >= ANGLES)
			angle -= ANGLES;
		Thrust (angle,controly*BACKMOVESCALE);		// move backwards
	}

	if (gamestate.victoryflag)		// watching the BJ actor
		return;

//
// calculate total move
//
	playerxmove = player->x - oldx;
	playerymove = player->y - oldy;
}

/*
=============================================================================

					STATUS WINDOW STUFF

=============================================================================
*/


/*
==================
=
= StatusDrawPic
=
==================
*/

void StatusDrawPic (unsigned x, unsigned y, unsigned picnum)
{
	unsigned	temp;

	temp = bufferofs;
	bufferofs = 0;

	bufferofs = PAGE1START+(200-STATUSLINES)*SCREENWIDTH;
	LatchDrawPic (x,y,picnum);
	bufferofs = PAGE2START+(200-STATUSLINES)*SCREENWIDTH;
	LatchDrawPic (x,y,picnum);
	bufferofs = PAGE3START+(200-STATUSLINES)*SCREENWIDTH;
	LatchDrawPic (x,y,picnum);

	bufferofs = temp;
}


/*
==================
=
= DrawFace
=
==================
*/

void DrawFace (void)
{
	if (gamestate.health)
	{
		if (godmode)
			StatusDrawPic (17,4,GODMODEFACE1PIC+gamestate.faceframe);
		else
            StatusDrawPic (17,4,FACE1APIC+3*((100-gamestate.health)/16)+gamestate.faceframe);
	}
	else
	{
        if (LastAttacker->obclass == needleobj)
            StatusDrawPic (17,4,MUTANTBJPIC);
        else
            StatusDrawPic (17,4,FACE8APIC);
	}
}


/*
===============
=
= UpdateFace
=
= Calls draw face if time to change
=
===============
*/

#define FACETICS	70

int	facecount;

void	UpdateFace (void)
{

	if (SD_SoundPlaying() == GETGATLINGSND)
	  return;

	facecount += tics;
	if (facecount > US_RndT())
	{
		gamestate.faceframe = (US_RndT()>>6);
		if (gamestate.faceframe==3)
			gamestate.faceframe = 1;

		facecount = 0;
		DrawFace ();
	}
}



/*
===============
=
= LatchNumber
=
= right justifies and pads with blanks
=
===============
*/

void	LatchNumber (int x, int y, int width, long number)
{
	unsigned	length,c;
	char	str[20];

	ltoa (number,str,10);

	length = strlen (str);

	while (length<width)
	{
		StatusDrawPic (x,y,N_BLANKPIC);
		x++;
		width--;
	}

	c= length <= width ? 0 : length-width;

	while (c<length)
	{
		StatusDrawPic (x,y,str[c]-'0'+ N_0PIC);
		x++;
		c++;
	}
}


/*
===============
=
= DrawHealth
=
===============
*/

void	DrawHealth (void)
{
	LatchNumber (21,16,3,gamestate.health);
}


/*
===============
=
= TakeDamage
=
===============
*/

void	TakeDamage (int points,objtype *attacker)
{
	LastAttacker = attacker;

	if (gamestate.victoryflag)
		return;
	if (gamestate.difficulty==gd_baby)
	  points>>=2;

	if (!godmode)
		gamestate.health -= points;

	if (gamestate.health<=0)
	{
		gamestate.health = 0;
		playstate = ex_died;
		killerobj = attacker;
	}

	StartDamageFlash (points);

	gotgatgun=0;

	DrawHealth ();
	DrawFace ();

	//
	// MAKE BJ'S EYES BUG IF MAJOR DAMAGE!
	//

	if (points > 30 && gamestate.health!=0 && !godmode)
	{
		StatusDrawPic (17,4,BJOUCHPIC);
		facecount = 0;
	}
}


/*
===============
=
= HealSelf
=
===============
*/

void	HealSelf (int points)
{
	gamestate.health += points;
	if (gamestate.health>100)
		gamestate.health = 100;

	DrawHealth ();
	gotgatgun = 0;	// JR
	DrawFace ();
}


//===========================================================================


/*
===============
=
= DrawLevel
=
===============
*/

void	DrawLevel (void)
{
    if (gamestate.mapon <= 2)
    {
        LatchNumber (4,16,1,gamestate.mapon+1);
        LatchNumber (2,16,1,gamestate.episode+1);
    }
    else if (gamestate.mapon <= 6)
    {
        LatchNumber (4,16,1,gamestate.mapon-2);
        LatchNumber (2,16,1,gamestate.episode+2);
    }
    else if (gamestate.mapon <= 10)
    {
        LatchNumber (4,16,1,gamestate.mapon-6);
        LatchNumber (2,16,1,gamestate.episode+3);
    }
    else if (gamestate.mapon <= 15)
    {
        LatchNumber (4,16,1,gamestate.mapon-10);
        LatchNumber (2,16,1,gamestate.episode+4);
    }
    else if (gamestate.mapon <= 21)
    {
        LatchNumber (4,16,1,gamestate.mapon-15);
        LatchNumber (2,16,1,gamestate.episode+5);
    }
    else if (gamestate.mapon <= 27)
    {
        LatchNumber (4,16,1,gamestate.mapon-21);
        LatchNumber (2,16,1,gamestate.episode+6);
    }
    if (gamestate.mapon == 28)
    {
        LatchNumber (4,16,1,gamestate.mapon-13);
        LatchNumber (2,16,1,gamestate.episode+3);
    }
    else if (gamestate.mapon == 29)
    {
        LatchNumber (4,16,1,gamestate.mapon-12);
        LatchNumber (2,16,1,gamestate.episode+6);
    }
}

//===========================================================================


/*
===============
=
= DrawLives
=
===============
*/

void	DrawLives (void)
{
	LatchNumber (14,16,1,gamestate.lives);
}


/*
===============
=
= GiveExtraMan
=
===============
*/

void	GiveExtraMan (void)
{
	if (gamestate.lives<9)
		gamestate.lives++;
	DrawLives ();
	SD_PlaySound (BONUS1UPSND);
}

//===========================================================================

/*
===============
=
= DrawScore
=
===============
*/

void	DrawScore (void)
{
	LatchNumber (6,16,6,gamestate.score);
}

/*
===============
=
= GivePoints
=
===============
*/

void	GivePoints (long points)
{
	gamestate.score += points;
	while (gamestate.score >= gamestate.nextextra)
	{
		gamestate.nextextra += EXTRAPOINTS;
		GiveExtraMan ();
	}
	DrawScore ();
}

//===========================================================================

/*
==================
=
= DrawWeapon
=
==================
*/

void DrawWeapon (void)
{
	StatusDrawPic (32,8,KNIFEPIC+gamestate.weapon);
}


/*
==================
=
= DrawKeys
=
==================
*/

void DrawKeys (void)
{
	if (gamestate.keys & 1)
		StatusDrawPic (30,4,GOLDKEYPIC);
	else
		StatusDrawPic (30,4,NOKEYPIC);

	if (gamestate.keys & 2)
		StatusDrawPic (30,20,SILVERKEYPIC);
	else
		StatusDrawPic (30,20,NOKEYPIC);
}



/*
==================
=
= GiveWeapon
=
==================
*/

void GiveWeapon (int weapon)
{
	switch (weapon)
	{
	case wp_machinegun:
	    GiveAmmo (6);
		break;
	case wp_chaingun:
	    GiveAmmo (6);
		break;
	case wp_flamethrower:
	    GiveFuel (6);
		break;
	case wp_rocketlauncher:
	    GiveRocket (6);
		break;
	}

	if ((gamestate.bestweapon & (1<<weapon)) == 0)
	{
		gamestate.bestweapon |= (1<<weapon);
		gamestate.weapon = gamestate.chosenweapon = weapon;
	}
	DrawWeapon ();
    DrawAmmo ();
}


//===========================================================================

/*
===============
=
= DrawAmmo
=
===============
*/

void	DrawAmmo (void)
{
    switch (gamestate.chosenweapon)
    {
	case wp_pistol:
		LatchNumber (26,16,3,gamestate.ammo);
		break;
	case wp_machinegun:
		LatchNumber (26,16,3,gamestate.ammo);
		break;
	case wp_chaingun:
		LatchNumber (26,16,3,gamestate.ammo);
		break;
    case wp_flamethrower:
        LatchNumber (26,16,3,gamestate.fuel);
        break;
    case wp_rocketlauncher:
        LatchNumber (26,16,3,gamestate.rockets);
        break;
    case wp_knife:
        LatchNumber (26,16,3,0);
        break;
    }
}

/*
===============
=
= GiveAmmo
=
===============
*/

void	GiveAmmo (int ammo)
{
	if (!gamestate.ammo)				// knife was out
	{
		if (!gamestate.attackframe)
		{
			gamestate.weapon = gamestate.chosenweapon;
			DrawWeapon ();
		}
	}
	gamestate.ammo += ammo;
    if (gamestate.backpack == 2)
    {
        if (gamestate.ammo > 299)
            gamestate.ammo = 299;
    }
    else if (gamestate.backpack == 1)
    {
        if (gamestate.ammo > 199)
            gamestate.ammo = 199;
    }
    else
    {
        if (gamestate.ammo > 99)
            gamestate.ammo = 99;
    }
	DrawAmmo ();
}

/*
===============
=
= GiveFuel
=
===============
*/

void	GiveFuel (int fuel)
{
	if (!gamestate.fuel)				// knife was out
	{
		if (!gamestate.attackframe)
		{
			gamestate.weapon = gamestate.chosenweapon;
			DrawWeapon ();
		}
	}
	gamestate.fuel += fuel;
        if (gamestate.fuel > 99)
            gamestate.fuel = 99;
	DrawAmmo ();
}

/*
===============
=
= GiveRocket
=
===============
*/

void	GiveRocket (int rocket)
{
	if (!gamestate.rockets)				// knife was out
	{
		if (!gamestate.attackframe)
		{
			gamestate.weapon = gamestate.chosenweapon;
			DrawWeapon ();
		}
	}
	gamestate.rockets += rocket;
        if (gamestate.rockets > 99)
            gamestate.rockets = 99;
	DrawAmmo ();
}

//===========================================================================

/*
==================
=
= GiveKey
=
==================
*/

void GiveKey (int key)
{
	gamestate.keys |= (1<<key);
	DrawKeys ();
}



/*
=============================================================================

							MOVEMENT

=============================================================================
*/


/*
===================
=
= GetBonus
=
===================
*/
void GetBonus (statobj_t *check)
{
	switch (check->itemnumber)
	{
	case	bo_alpo:
		if (gamestate.health == 100)
			return;

		SD_PlaySound (HEALTH1SND);
		HealSelf (4);
		break;

	case	bo_food:
		if (gamestate.health == 100)
			return;

		SD_PlaySound (HEALTH1SND);
		HealSelf (10);
		break;

	case	bo_firstaid:
		if (gamestate.health == 100)
			return;

		SD_PlaySound (HEALTH2SND);
		HealSelf (25);
		break;

	case	bo_key1:
	case	bo_key2:
		GiveKey (check->itemnumber - bo_key1);
		SD_PlaySound (GETKEYSND);
		break;

	case	bo_cross:
		SD_PlaySound (BONUS1SND);
		GivePoints (100);
		gamestate.treasurecount++;
		break;

	case	bo_chalice:
		SD_PlaySound (BONUS2SND);
		GivePoints (500);
		gamestate.treasurecount++;
		break;

	case	bo_bible:
		SD_PlaySound (BONUS3SND);
		GivePoints (1000);
		gamestate.treasurecount++;
		break;

	case	bo_crown:
		SD_PlaySound (BONUS4SND);
		GivePoints (5000);
		gamestate.treasurecount++;
		break;

	case	bo_clip:
		if ((gamestate.ammo == 299) && (gamestate.backpack == 2))
			return;
		if ((gamestate.ammo == 199) && (gamestate.backpack == 1))
			return;
		if ((gamestate.ammo == 99) && (gamestate.backpack == 0))
		    return;

		SD_PlaySound (GETAMMOSND);
		GiveAmmo (8);
		break;
	case	bo_clip2:
		if ((gamestate.ammo == 299) && (gamestate.backpack == 2))
			return;
		if ((gamestate.ammo == 199) && (gamestate.backpack == 1))
			return;
		if ((gamestate.ammo == 99) && (gamestate.backpack == 0))
		    return;

		SD_PlaySound (GETAMMOSND);
		GiveAmmo (4);
		break;

	case	bo_25clip:
		if ((gamestate.ammo == 299) && (gamestate.backpack == 2))
			return;
		if ((gamestate.ammo == 199) && (gamestate.backpack == 1))
			return;
		if ((gamestate.ammo == 99) && (gamestate.backpack == 0))
		    return;

		SD_PlaySound (GETAMMOSND);
		GiveAmmo (25);
		break;

    case    bo_fuel:
		if (gamestate.fuel == 99)
		    return;

		SD_PlaySound (GETAMMOSND);
		GiveFuel (14);
		break;

    case    bo_rocket:
		if (gamestate.rockets == 99)
		    return;

		SD_PlaySound (GETAMMOSND);
		GiveRocket (5);
		break;

    case    bo_backpack:
        if (gamestate.backpack < 2)
            gamestate.backpack++;

        SD_PlaySound (GETAMMOSND);
        GiveAmmo (25);
        GiveFuel (2);
        GiveRocket (5);
        break;

	case	bo_machinegun:
		SD_PlaySound (GETMACHINESND);
		GiveWeapon (wp_machinegun);
		break;

	case	bo_chaingun:
		SD_PlaySound (GETGATLINGSND);
		GiveWeapon (wp_chaingun);

		StatusDrawPic (17,4,GOTGATLINGPIC);
		facecount = 0;
		gotgatgun = 1;
		break;

	case	bo_flamethrower:
		SD_PlaySound (GETMACHINESND);
		GiveWeapon (wp_flamethrower);
		break;

	case	bo_rocketlauncher:
		SD_PlaySound (GETGATLINGSND);
		GiveWeapon (wp_rocketlauncher);

		StatusDrawPic (17,4,GOTGATLINGPIC);
		facecount = 0;
		gotgatgun = 1;
		break;

	case	bo_fullheal:
		SD_PlaySound (BONUS1UPSND);
		HealSelf (99);
		GiveAmmo (25);
		GiveFuel (2);
		GiveRocket (5);
		GiveExtraMan ();
		gamestate.treasurecount++;
		break;
	}
	StartBonusFlash ();
	check->shapenum = -1;			// remove from list
}


/*
===================
=
= TryMove
=
= returns true if move ok
= debug: use pointers to optimize
===================
*/

boolean TryMove (objtype *ob)
{
	int			xl,yl,xh,yh,x,y;
	objtype		*check;
	long		deltax,deltay;

	xl = (ob->x-PLAYERSIZE) >>TILESHIFT;
	yl = (ob->y-PLAYERSIZE) >>TILESHIFT;

	xh = (ob->x+PLAYERSIZE) >>TILESHIFT;
	yh = (ob->y+PLAYERSIZE) >>TILESHIFT;

//
// check for solid walls
//
	for (y=yl;y<=yh;y++)
		for (x=xl;x<=xh;x++)
		{
			check = actorat[x][y];
			if (check && check<objlist)
				return false;
		}

//
// check for actors
//
	if (yl>0)
		yl--;
	if (yh<MAPSIZE-1)
		yh++;
	if (xl>0)
		xl--;
	if (xh<MAPSIZE-1)
		xh++;

	for (y=yl;y<=yh;y++)
		for (x=xl;x<=xh;x++)
		{
			check = actorat[x][y];
			if (check > objlist
			&& (check->flags & FL_SHOOTABLE) )
			{
				deltax = ob->x - check->x;
				if (deltax < -MINACTORDIST || deltax > MINACTORDIST)
					continue;
				deltay = ob->y - check->y;
				if (deltay < -MINACTORDIST || deltay > MINACTORDIST)
					continue;

				return false;
			}
		}

	return true;
}


/*
===================
=
= ClipMove
=
===================
*/

void ClipMove (objtype *ob, long xmove, long ymove)
{
	long	basex,basey;

	basex = ob->x;
	basey = ob->y;

	ob->x = basex+xmove;
	ob->y = basey+ymove;
	if (TryMove (ob))
		return;

	if (noclip && ob->x > 2*TILEGLOBAL && ob->y > 2*TILEGLOBAL &&
	ob->x < (((long)(mapwidth-1))<<TILESHIFT)
	&& ob->y < (((long)(mapheight-1))<<TILESHIFT) )
		return;		// walk through walls

	if (!SD_SoundPlaying())
		SD_PlaySound (HITWALLSND);

	ob->x = basex+xmove;
	ob->y = basey;
	if (TryMove (ob))
		return;

	ob->x = basex;
	ob->y = basey+ymove;
	if (TryMove (ob))
		return;

	ob->x = basex;
	ob->y = basey;
}

//==========================================================================

/*
===================
=
= Thrust
=
===================
*/

void Thrust (int angle, long speed)
{
	long xmove,ymove;
	long	slowmax;
	unsigned	offset;

	thrustspeed += speed;
//
// moving bounds speed
//
	if (speed >= MINDIST*2)
		speed = MINDIST*2-1;

	xmove = FixedByFrac(speed,costable[angle]);
	ymove = -FixedByFrac(speed,sintable[angle]);

	ClipMove(player,xmove,ymove);

	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;

	offset = farmapylookup[player->tiley]+player->tilex;
	player->areanumber = *(mapsegs[0] + offset) -AREATILE;
}


/*
=============================================================================

								ACTIONS

=============================================================================
*/


/*
===============
=
= Cmd_Fire
=
===============
*/

void Cmd_Fire (void)
{
	buttonheld[bt_attack] = true;

	gamestate.weaponframe = 0;

	player->state = &s_attack;

	gamestate.attackframe = 0;
	gamestate.attackcount =
		attackinfo[gamestate.weapon][gamestate.attackframe].tics;
	gamestate.weaponframe =
		attackinfo[gamestate.weapon][gamestate.attackframe].frame;
}

//===========================================================================

/*
===============
=
= Cmd_Use
=
===============
*/

void Cmd_Use (void)
{
	objtype 	*check;
	int			checkx,checky,doornum,dir;
	boolean		elevatorok;


//
// find which cardinal direction the player is facing
//
	if (player->angle < ANGLES/8 || player->angle > 7*ANGLES/8)
	{
		checkx = player->tilex + 1;
		checky = player->tiley;
		dir = di_east;
		elevatorok = true;
	}
	else if (player->angle < 3*ANGLES/8)
	{
		checkx = player->tilex;
		checky = player->tiley-1;
		dir = di_north;
		elevatorok = false;
	}
	else if (player->angle < 5*ANGLES/8)
	{
		checkx = player->tilex - 1;
		checky = player->tiley;
		dir = di_west;
		elevatorok = true;
	}
	else
	{
		checkx = player->tilex;
		checky = player->tiley + 1;
		dir = di_south;
		elevatorok = false;
	}

	doornum = tilemap[checkx][checky];
	if (*(mapsegs[1]+farmapylookup[checky]+checkx) == PUSHABLETILE)
	{
	//
	// pushable wall
	//

		PushWall (checkx,checky,dir);
		return;
	}
	if (!buttonheld[bt_use] && doornum == ELEVATORTILE && elevatorok)
	{
	//
	// use elevator
	//
		buttonheld[bt_use] = true;

		tilemap[checkx][checky]++;		// flip switch
		if (*(mapsegs[0]+farmapylookup[player->tiley]+player->tilex) == ALTELEVATORTILE)
			playstate = ex_secretlevel;
		else
			playstate = ex_completed;
		SD_PlaySound (LEVELDONESND);
		SD_WaitSoundDone();
	}
	else if (!buttonheld[bt_use] && doornum & 0x80)
	{
		buttonheld[bt_use] = true;
		OperateDoor (doornum & ~0x80);
	}
	else
		SD_PlaySound (DONOTHINGSND);

}

/*
=============================================================================

						   PLAYER CONTROL

=============================================================================
*/



/*
===============
=
= SpawnPlayer
=
===============
*/

void SpawnPlayer (int tilex, int tiley, int dir)
{
	player->obclass = playerobj;
	player->active = true;
	player->tilex = tilex;
	player->tiley = tiley;
	player->areanumber =
		*(mapsegs[0] + farmapylookup[player->tiley]+player->tilex);
	player->x = ((long)tilex<<TILESHIFT)+TILEGLOBAL/2;
	player->y = ((long)tiley<<TILESHIFT)+TILEGLOBAL/2;
	player->state = &s_player;
	player->angle = (1-dir)*90;
	if (player->angle<0)
		player->angle += ANGLES;
	player->flags = FL_NEVERMARK;
	Thrust (0,0);				// set some variables

	InitAreas ();
}


//===========================================================================

/*
===============
=
= T_KnifeAttack
=
= Update player hands, and try to do damage when the proper frame is reached
=
===============
*/

void	KnifeAttack (objtype *ob)
{
	objtype *check,*closest;
	long	dist;

	SD_PlaySound (ATKKNIFESND);
// actually fire
	dist = 0x7fffffff;
	closest = NULL;
	for (check=ob->next ; check ; check=check->next)
		if ( (check->flags & FL_SHOOTABLE)
		&& (check->flags & FL_VISABLE)
		&& abs (check->viewx-centerx) < shootdelta
		)
		{
			if (check->transx < dist)
			{
				dist = check->transx;
				closest = check;
			}
		}

	if (!closest || dist> 0x18000l)
	{
	// missed

		return;
	}

// hit something
	DamageActor (closest,US_RndT() >> 4);
}

/*
==========================
=
= T_FlameThrowerAttack
=
==========================
*/

void FlameThrowerAttack (objtype *ob)
{
   if (!gamestate.fuel)
       return;

   SD_PlaySound (FLAMETHROWERSND); //change to any sound you have

   GetNewActor ();
   new->state = &s_fire1;
   new->ticcount = 1;

   new->tilex = ob->tilex;
   new->tiley = ob->tiley;
   new->x = ob->x;
   new->y = ob->y;

   new->obclass = fireobj;
   new->dir = nodir;
   new->angle = ob->angle;
   new->speed = 0x2000l;
   new->flags = FL_NEVERMARK;
   new->active = true;
}

/*
==========================
=
= T_RocketLauncherAttack
=
==========================
*/

void RocketLauncherAttack (objtype *ob)
{
   if (!gamestate.rockets)
       return;

   SD_PlaySound (MISSILEFIRESND); //change to any sound you have

   GetNewActor ();
   new->state = &s_rocket;
   new->ticcount = 1;

   new->tilex = ob->tilex;
   new->tiley = ob->tiley;
   new->x = ob->x;
   new->y = ob->y;

   new->obclass = rocketobj;
   new->dir = nodir;
   new->angle = ob->angle;
   new->speed = 0x4200l;
   new->flags = FL_NEVERMARK;
   new->active = true;
}

void	GunAttack (objtype *ob)
{
	objtype *check,*closest,*oldclosest;
	int		damage;
	int		dx,dy,dist;
	long	viewdist;

	switch (gamestate.weapon)
	{
	case wp_pistol:
		SD_PlaySound (ATKPISTOLSND);
		break;
	case wp_machinegun:
		SD_PlaySound (ATKMACHINEGUNSND);
		break;
	case wp_chaingun:
		SD_PlaySound (ATKGATLINGSND);
		break;
    case wp_flamethrower:
        FlameThrowerAttack (ob);
        madenoise = true;
        return;
    case wp_rocketlauncher:
        RocketLauncherAttack (ob);
        madenoise = true;
        return;
	}

	madenoise = true;

//
// find potential targets
//
	viewdist = 0x7fffffffl;
	closest = NULL;

	while (1)
	{
		oldclosest = closest;

		for (check=ob->next ; check ; check=check->next)
			if ( (check->flags & FL_SHOOTABLE)
			&& (check->flags & FL_VISABLE)
			&& abs (check->viewx-centerx) < shootdelta
			)
			{
				if (check->transx < viewdist)
				{
					viewdist = check->transx;
					closest = check;
				}
			}

		if (closest == oldclosest)
			return;						// no more targets, all missed

	//
	// trace a line from player to enemy
	//
		if (CheckLine(closest))
			break;

	}

//
// hit something
//
	dx = abs(closest->tilex - player->tilex);
	dy = abs(closest->tiley - player->tiley);
	dist = dx>dy ? dx:dy;

	if (dist<2)
		damage = US_RndT() / 4;
	else if (dist<4)
		damage = US_RndT() / 6;
	else
	{
		if ( (US_RndT() / 12) < dist)		// missed
			return;
		damage = US_RndT() / 6;
	}

	DamageActor (closest,damage);
}

//===========================================================================

/*
===============
=
= T_Attack
=
===============
*/

void	T_Attack (objtype *ob)
{
	struct	atkinf	*cur;
    int i;

	UpdateFace ();

	if ( buttonstate[bt_use] && !buttonheld[bt_use] )
		buttonstate[bt_use] = false;

	if ( buttonstate[bt_attack] && !buttonheld[bt_attack])
		buttonstate[bt_attack] = false;

	ControlMovement (ob);

	plux = player->x >> UNSIGNEDSHIFT;			// scale to fit in unsigned
	pluy = player->y >> UNSIGNEDSHIFT;
	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;

//
// change frame and fire
//
	gamestate.attackcount -= tics;
	while (gamestate.attackcount <= 0)
	{
		cur = &attackinfo[gamestate.weapon][gamestate.attackframe];
		switch (cur->attack)
		{
		case -1:
			ob->state = &s_player;
			if (!CheckAmmo (gamestate.weapon) && gamestate.chosenweapon != wp_knife)
            {
                for(i=0; i<NUMWEAPONS; i++)
                {
                    if(CheckAmmo (i) && gamestate.bestweapon & (1<<i))
                    {
                        gamestate.weapon = gamestate.chosenweapon = i;
                    }
                }
                DrawWeapon ();
                DrawAmmo ();
            }
			else
			{
				if (gamestate.weapon != gamestate.chosenweapon)
				{
                    if(CheckAmmo(gamestate.chosenweapon))
                    {
                        gamestate.weapon = gamestate.chosenweapon;
                    }
                    DrawWeapon ();
				}
			}
			gamestate.attackframe = gamestate.weaponframe = 0;
			return;
		case 4:
			if (!CheckAmmo(gamestate.weapon))
				break;
			if (buttonstate[bt_attack])
				gamestate.attackframe -= 2;
		case 1:
			if (!CheckAmmo(wp_chaingun) && gamestate.weapon == wp_chaingun)
			{	// can only happen with chain gun
				gamestate.attackframe++;
				break;
			}
			if (!CheckAmmo(wp_flamethrower) && gamestate.weapon == wp_flamethrower)
			{	// can only happen with flamethrower
				gamestate.attackframe++;
				break;
			}
            if (CheckAmmo(gamestate.weapon))
            {
                GunAttack (ob);
                switch (gamestate.weapon)
                {
                    case wp_pistol:
                        gamestate.ammo--;
                        break;
                    case wp_machinegun:
                        gamestate.ammo--;
                        break;
                    case wp_chaingun:
                        gamestate.ammo--;
                        break;
                    case wp_flamethrower:
                        gamestate.fuel--;
                        break;
                    case wp_rocketlauncher:
                        gamestate.rockets--;
                        break;
                }
                DrawAmmo ();
                break;
            }
            else
            {
                return;
            }
		case 2:
			KnifeAttack (ob);
			break;

		case 3:
			if (CheckAmmo(gamestate.weapon) && buttonstate[bt_attack])
				gamestate.attackframe -= 2;
			break;
		}
		gamestate.attackcount += cur->tics;
		gamestate.attackframe++;
		gamestate.weaponframe =
			attackinfo[gamestate.weapon][gamestate.attackframe].frame;
	}
}



//===========================================================================

/*
===============
=
= T_Player
=
===============
*/

void	T_Player (objtype *ob)
{
	UpdateFace ();
	CheckWeaponChange ();

	if ( buttonstate[bt_use] )
		Cmd_Use ();

	if ( buttonstate[bt_attack] && !buttonheld[bt_attack])
		Cmd_Fire ();

	ControlMovement (ob);

	plux = player->x >> UNSIGNEDSHIFT;			// scale to fit in unsigned
	pluy = player->y >> UNSIGNEDSHIFT;
	player->tilex = player->x >> TILESHIFT;		// scale to tile values
	player->tiley = player->y >> TILESHIFT;
}


